"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const fs = require("fs");
const path = require("path");
const Mustache = require("mustache");
const basic_1 = require("../templates/basic");
const utils_1 = require("../utils");
const types_1 = require("../types");
const cli_ux_1 = require("cli-ux");
const util_1 = require("util");
const propmts = require("prompts");
const readPgkUp = require("read-pkg-up");
const emoji = require('emoji-random');
class Generate extends command_1.Command {
    async run() {
        const { flags } = this.parse(Generate);
        cli_ux_1.default.action.start("effe is generating");
        let pJson = null;
        try {
            const res = await readPgkUp();
            pJson = res.pkg;
        }
        catch (error) {
            throw Error("Unable to find a package.json");
        }
        if (util_1.isNullOrUndefined(pJson)) {
            throw Error("Unable to find a package.json");
        }
        if (pJson.effe === undefined) {
            let packageManagerSelected = flags.packageManagerSelected;
            if (!packageManagerSelected) {
                let responses = await propmts({
                    name: "packagemanagerSelected",
                    message: `from where people can download ${pJson.name}?`,
                    type: "select",
                    choices: [{
                            title: "npm", value: "npm",
                        }, {
                            title: "yarn", value: "yarn",
                        }, {
                            title: "none of these", value: "none of these",
                        }]
                });
                packageManagerSelected = responses.packagemanagerSelected != "none of these" ? responses.packagemanagerSelected : null;
            }
            try {
                pJson = await utils_1.writeBasicConfig(pJson, packageManagerSelected);
            }
            catch (error) {
                throw Error(error);
            }
        }
        let licenseDescription = null;
        if (pJson.license && utils_1.getLicense(pJson.license)) {
            licenseDescription = Mustache.render(utils_1.getLicense(pJson.license), {
                author: pJson.author ? pJson.author.name : "Author empty"
            });
        }
        const randomEmoji = emoji.random();
        const dependenciesPJsonMetadata = await utils_1.fetchPackagesJson(Object.keys(pJson.dependencies || []));
        const displayableDependencies = dependenciesPJsonMetadata.map(d => ({
            dependencyName: d.name,
            dependencyVersion: d.version,
            dependencyDescription: d.description,
        }));
        let installationInstructions = null;
        if (pJson.effe.installfrom) {
            switch (pJson.effe.installfrom) {
                case types_1.PackageManagerSupported.npm:
                    installationInstructions = "npm install " + pJson.name;
                    break;
                case types_1.PackageManagerSupported.yarn:
                    installationInstructions = "yarn add " + pJson.name;
                    break;
                default:
                    installationInstructions = "effe don't know " + pJson.effe.installfrom;
                    break;
            }
        }
        let testInstructionDescription = null;
        if (pJson.scripts && pJson.scripts.test && pJson.effe.testInstructions) {
            testInstructionDescription = pJson.scripts.test;
        }
        try {
            const rendered = Mustache.render(basic_1.basicTemplate, {
                emoji: pJson.effe.emoji ? randomEmoji : null,
                mentionme: pJson.effe.mentionme,
                name: pJson.name,
                description: pJson.description,
                author: pJson.author ? pJson.author.name : "Author empty",
                testInstruction: testInstructionDescription,
                dependencies: pJson.effe.tecnhologies ? displayableDependencies : [],
                howtocontribute: pJson.effe ? pJson.effe.howtocontribute : false,
                license: pJson.license,
                licenseDescription: licenseDescription,
                packagemanager: pJson.effe.installfrom ? pJson.effe.installfrom : null,
                installationInstructions: installationInstructions,
                linkedin: pJson.effe.linkedin,
                twitter: pJson.effe.twitter,
                twitt: pJson.effe.twitt,
                githubrepo: pJson.effe.githubrepo
            });
            const readmePath = path.dirname("package.json");
            fs.writeFile(`${readmePath}/README.md`, rendered, err => {
                if (err) {
                    throw Error(err.stack);
                }
                else {
                    cli_ux_1.default.action.stop(`tada -> ${readmePath}/README.md`);
                }
            });
        }
        catch (error) {
            throw Error("Unable to render your new README.md: " + error);
        }
    }
}
Generate.description = 'generate README.md from package.json';
Generate.flags = {};
Generate.args = [{ name: 'file' }];
exports.default = Generate;
